@@ -1,25 +1,13 @@
-/* This file is part of the KDE libraries
-    Copyright (c) 1997-1999 Matthias Kalle Dalheimer <kalle@kde.org>
-    Copyright (c) 1997-2000 Matthias Ettrich <ettrich@troll.no>
-    Copyright (c) 1998-2005 Stephan Kulow <coolo@kde.org>
-    Copyright (c) 1999-2004 Waldo Bastian <bastian@kde.org>
-    Copyright (c) 2001-2005 Lubos Lunak <l.lunak@kde.org>
-    Copyright (C) 2008 Aaron Seigo <aseigo@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 1997-1999 Matthias Kalle Dalheimer <kalle@kde.org>
+    SPDX-FileCopyrightText: 1997-2000 Matthias Ettrich <ettrich@troll.no>
+    SPDX-FileCopyrightText: 1998-2005 Stephan Kulow <coolo@kde.org>
+    SPDX-FileCopyrightText: 1999-2004 Waldo Bastian <bastian@kde.org>
+    SPDX-FileCopyrightText: 2001-2005 Lubos Lunak <l.lunak@kde.org>
+    SPDX-FileCopyrightText: 2008 Aaron Seigo <aseigo@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef _KTOOLINVOCATION_H
@@ -27,12 +15,15 @@
 
 #include <kservice_export.h>
 
-#include <QtCore/QObject>
-#include <QtCore/QByteArray>
-#include <QtCore/QStringList>
+#include <QByteArray>
+#include <QExplicitlySharedDataPointer>
+#include <QObject>
+#include <QStringList>
 
 class QUrl;
 class KToolInvocationPrivate;
+class KService;
+typedef QExplicitlySharedDataPointer<KService> KServicePtr;
 
 /**
  * @class KToolInvocation ktoolinvocation.h <KToolInvocation>
@@ -71,10 +62,10 @@ class KToolInvocationPrivate;
  */
 class KSERVICE_EXPORT KToolInvocation : public QObject
 {
-
     Q_OBJECT
 private:
     KToolInvocation();
+
 public:
     // @internal
     ~KToolInvocation();
@@ -82,6 +73,7 @@ public:
 
 public Q_SLOTS:
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Convenience method; invokes the standard email application.
      *
@@ -93,9 +85,11 @@ public Q_SLOTS:
      * @deprecated since 5.0, use QDesktopServices::openUrl(mailtoURL),
      * using QUrl::setPath(address) and a query item of "subject" for the subject.
      */
-    KSERVICE_DEPRECATED static void invokeMailer(const QString &address, const QString &subject,
-                             const QByteArray &startup_id = QByteArray());
+    KSERVICE_DEPRECATED_VERSION(5, 0, "Use QDesktopServices::openUrl(mailtoURL), using QUrl::setPath(address) and a query item of \"subject\" for the subject")
+    static void invokeMailer(const QString &address, const QString &subject, const QByteArray &startup_id = QByteArray());
+#endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Invokes the standard email application.
      *
@@ -106,8 +100,9 @@ public Q_SLOTS:
      *           The default is false; do not honor requests for attachments.
      * @deprecated since 5.0, use QDesktopServices::openUrl(mailtoURL)
      */
-    KSERVICE_DEPRECATED static void invokeMailer(const QUrl &mailtoURL, const QByteArray &startup_id = QByteArray(),
-                             bool allowAttachments = false);
+    KSERVICE_DEPRECATED_VERSION(5, 0, "Use QDesktopServices::openUrl(const QUrl&)")
+    static void invokeMailer(const QUrl &mailtoURL, const QByteArray &startup_id = QByteArray(), bool allowAttachments = false);
+#endif
 
     /**
      * Convenience method; invokes the standard email application.
@@ -124,16 +119,20 @@ public Q_SLOTS:
      * @param startup_id for app startup notification, "0" for none,
      *           "" ( empty string ) is the default
      */
-    static void invokeMailer(const QString &to, const QString &cc, const QString &bcc,
-                             const QString &subject, const QString &body,
+    static void invokeMailer(const QString &to,
+                             const QString &cc,
+                             const QString &bcc,
+                             const QString &subject,
+                             const QString &body,
                              const QString &messageFile = QString(),
                              const QStringList &attachURLs = QStringList(),
                              const QByteArray &startup_id = QByteArray());
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Invokes the user's preferred browser.
      * Note that you should only do this when you know for sure that the browser can
-     * handle the URL (i.e. its mimetype). In doubt, if the URL can point to an image
+     * handle the URL (i.e. its MIME type). In doubt, if the URL can point to an image
      * or anything else than HTML, prefer to use new KRun( url ).
      *
      * See also <a
@@ -145,10 +144,25 @@ public Q_SLOTS:
      *           "" ( empty string ) is the default
      * @deprecated since 5.0, use QDesktopServices::openUrl(url)
      */
-    KSERVICE_DEPRECATED static void invokeBrowser(const QString &url,
-                                                  const QByteArray &startup_id = QByteArray());
+    KSERVICE_DEPRECATED_VERSION(5, 0, "Use QDesktopServices::openUrl(const QUrl&)")
+    static void invokeBrowser(const QString &url, const QByteArray &startup_id = QByteArray());
+#endif
 
     /**
+     * Invokes the standard terminal application.
+     *
+     * @param command the command to execute, can be empty.
+     * @param envs ENV variables for the invoked terminal command
+     * @param workdir the initial working directory, can be empty.
+     * @param startup_id for app startup notification, "0" for none,
+     *           "" ( empty string ) is the default
+     * @since 5.73
+     */
+    static void invokeTerminal(const QString &command, const QStringList &envs, const QString &workdir = QString(), const QByteArray &startup_id = "");
+
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 79)
+    /**
+     *
      * Invokes the standard terminal application.
      *
      * @param command the command to execute, can be empty.
@@ -157,13 +171,27 @@ public Q_SLOTS:
      *           "" ( empty string ) is the default
      *
      * @since 4.1
+     * @deprecated since 5,79, use invokeTerminal(const QString &command, const QStringList &envs, const QString &workdir, const QByteArray &startup_id) instead
      */
-    static void invokeTerminal(const QString &command,
-                               const QString &workdir = QString(),
-                               const QByteArray &startup_id = "");
+    KSERVICE_DEPRECATED_VERSION(
+        5,
+        79,
+        "Use invokeTerminal(const QString &command, const QStringList &envs, const QString &workdir, const QByteArray &startup_id) instead")
+    static void invokeTerminal(const QString &command, const QString &workdir = QString(), const QByteArray &startup_id = "");
+#endif
+
+    /**
+     * Returns the configured default terminal application. This is compatible with
+     * the old config format from the KCM in Plasma < 5.21.
+     * @param command Command that should be executed in the terminal app
+     * @param workingDir Working directory for the terminal app.
+     * @return KServicePtr of terminal application.
+     * @since 5.78
+     */
+    static KServicePtr terminalApplication(const QString &command = QString(), const QString &workingDir = QString());
 
 public:
-
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Starts a service based on the (translated) name of the service.
      * E.g. "Web Browser"
@@ -173,9 +201,9 @@ public:
      * @param error On failure, error contains a description of the error
      *         that occurred. If the pointer is 0, the argument will be
      *         ignored
-     * @param serviceName On success, serviceName contains the DCOP name
+     * @param serviceName On success, serviceName contains the D-Bus name
      *         under which this service is available. If empty, the service does
-     *         not provide DCOP services. If the pointer is 0 the argument
+     *         not provide D-Bus services. If the pointer is 0 the argument
      *         will be ignored
      * @param pid On success, the process id of the new service will be written
      *        here. If the pointer is 0, the argument will be ignored.
@@ -183,14 +211,19 @@ public:
      *           "" ( empty string ) is the default
      * @param noWait if set, the function does not wait till the service is running.
      * @return an error code indicating success (== 0) or failure (> 0).
-     * @deprecated Use startServiceByDesktopName or startServiceByDesktopPath
+     * @deprecated Since 5.0, use startServiceByDesktopName or startServiceByDesktopPath
      */
-#ifndef KSERVICE_NO_DEPRECATED
-    KSERVICE_DEPRECATED static int startServiceByName(const QString &_name, const QString &URL,
-            QString *error = nullptr, QString *serviceName = nullptr, int *pid = nullptr,
-            const QByteArray &startup_id = QByteArray(), bool noWait = false);
+    KSERVICE_DEPRECATED_VERSION(5, 0, "Use KToolInvocation::startServiceByDesktopName(...) or KToolInvocation::startServiceByDesktopPath(...)")
+    static int startServiceByName(const QString &_name,
+                                  const QString &URL,
+                                  QString *error = nullptr,
+                                  QString *serviceName = nullptr,
+                                  int *pid = nullptr,
+                                  const QByteArray &startup_id = QByteArray(),
+                                  bool noWait = false);
 #endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Starts a service based on the (translated) name of the service.
      * E.g. "Web Browser"
@@ -200,9 +233,9 @@ public:
      * @param error On failure, @p error contains a description of the error
      *         that occurred. If the pointer is 0, the argument will be
      *         ignored
-     * @param serviceName On success, @p serviceName contains the DCOP name
+     * @param serviceName On success, @p serviceName contains the D-Bus name
      *         under which this service is available. If empty, the service does
-     *         not provide DCOP services. If the pointer is 0 the argument
+     *         not provide D-Bus services. If the pointer is 0 the argument
      *         will be ignored
      * @param pid On success, the process id of the new service will be written
      *        here. If the pointer is 0, the argument will be ignored.
@@ -210,14 +243,19 @@ public:
      *           "" ( empty string ) is the default
      * @param noWait if set, the function does not wait till the service is running.
      * @return an error code indicating success (== 0) or failure (> 0).
-     * @deprecated Use startServiceByDesktopName or startServiceByDesktopPath
+     * @deprecated Since 5.0, use startServiceByDesktopName or startServiceByDesktopPath
      */
-#ifndef KSERVICE_NO_DEPRECATED
-    KSERVICE_DEPRECATED static int startServiceByName(const QString &_name, const QStringList &URLs = QStringList(),
-            QString *error = nullptr, QString *serviceName = nullptr, int *pid = nullptr,
-            const QByteArray &startup_id = QByteArray(), bool noWait = false);
+    KSERVICE_DEPRECATED_VERSION(5, 0, "Use KToolInvocation::startServiceByDesktopName(...) or KToolInvocation::startServiceByDesktopPath(...)")
+    static int startServiceByName(const QString &_name,
+                                  const QStringList &URLs = QStringList(),
+                                  QString *error = nullptr,
+                                  QString *serviceName = nullptr,
+                                  int *pid = nullptr,
+                                  const QByteArray &startup_id = QByteArray(),
+                                  bool noWait = false);
 #endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 82)
     /**
      * Starts a service based on the desktop path of the service.
      * E.g. "Applications/konqueror.desktop" or "/home/user/bla/myfile.desktop"
@@ -227,9 +265,9 @@ public:
      * @param error On failure, @p error contains a description of the error
      *         that occurred. If the pointer is 0, the argument will be
      *         ignored
-     * @param serviceName On success, @p serviceName contains the DCOP name
+     * @param serviceName On success, @p serviceName contains the D-Bus name
      *         under which this service is available. If empty, the service does
-     *         not provide DCOP services. If the pointer is 0 the argument
+     *         not provide D-Bus services. If the pointer is 0 the argument
      *         will be ignored
      * @param pid On success, the process id of the new service will be written
      *        here. If the pointer is 0, the argument will be ignored.
@@ -240,12 +278,20 @@ public:
      *
      * @deprecated since 5.0 use QDBusConnectionInterface::startService("org.kde.serviceName"),
      *   to start a unique application in order to make D-Bus calls to it (after ensuring that
-     *   it installs a D-Bus org.kde.serviceName.service file). Otherwise just use QProcess or KRun.
+     *   it installs a D-Bus org.kde.serviceName.service file). Otherwise just use QProcess,
+     *   KIO::CommandLauncherJob or KIO::ApplicationLauncherJob.
      */
-    static int startServiceByDesktopPath(const QString &_name, const QString &URL,
-                                         QString *error = nullptr, QString *serviceName = nullptr, int *pid = nullptr,
-                                         const QByteArray &startup_id = QByteArray(), bool noWait = false);
+    KSERVICE_DEPRECATED_VERSION_BELATED(5, 82, 5, 0, "See API documentation")
+    static int startServiceByDesktopPath(const QString &_name,
+                                         const QString &URL,
+                                         QString *error = nullptr,
+                                         QString *serviceName = nullptr,
+                                         int *pid = nullptr,
+                                         const QByteArray &startup_id = QByteArray(),
+                                         bool noWait = false);
+#endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 82)
     /**
      * Starts a service based on the desktop path of the service.
      * E.g. "Applications/konqueror.desktop" or "/home/user/bla/myfile.desktop"
@@ -254,9 +300,9 @@ public:
      * @param URLs if not empty these URLs will be passed to the service
      * @param error On failure, @p error contains a description of the error
      *         that occurred. If the pointer is 0, the argument will be
-     *         ignored   * @param serviceName On success, @p serviceName contains the DCOP name
+     *         ignored   * @param serviceName On success, @p serviceName contains the D-Bus name
      *         under which this service is available. If empty, the service does
-     *         not provide DCOP services. If the pointer is 0 the argument
+     *         not provide D-Bus services. If the pointer is 0 the argument
      *         will be ignored
      * @param pid On success, the process id of the new service will be written
      *        here. If the pointer is 0, the argument will be ignored.
@@ -266,12 +312,20 @@ public:
      * @return an error code indicating success (== 0) or failure (> 0).
      * @deprecated since 5.0 use QDBusConnectionInterface::startService("org.kde.serviceName"),
      *   to start a unique application in order to make D-Bus calls to it (after ensuring that
-     *   it installs a D-Bus org.kde.serviceName.service file). Otherwise just use QProcess or KRun.
+     *   it installs a D-Bus org.kde.serviceName.service file). Otherwise just use QProcess,
+     *   KIO::CommandLauncherJob or KIO::ApplicationLauncherJob.
      */
-    static int startServiceByDesktopPath(const QString &_name, const QStringList &URLs = QStringList(),
-                                         QString *error = nullptr, QString *serviceName = nullptr, int *pid = nullptr,
-                                         const QByteArray &startup_id = QByteArray(), bool noWait = false);
+    KSERVICE_DEPRECATED_VERSION_BELATED(5, 82, 5, 0, "See API documentation")
+    static int startServiceByDesktopPath(const QString &_name,
+                                         const QStringList &URLs = QStringList(),
+                                         QString *error = nullptr,
+                                         QString *serviceName = nullptr,
+                                         int *pid = nullptr,
+                                         const QByteArray &startup_id = QByteArray(),
+                                         bool noWait = false);
+#endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 82)
     /**
      * Starts a service based on the desktop name of the service.
      * E.g. "konqueror"
@@ -293,12 +347,20 @@ public:
      * @return an error code indicating success (== 0) or failure (> 0).
      * @deprecated since 5.0 use QDBusConnectionInterface::startService("org.kde.serviceName"),
      *   to start a unique application in order to make D-Bus calls to it (after ensuring that
-     *   it installs a D-Bus org.kde.serviceName.service file). Otherwise just use QProcess or KRun.
+     *   it installs a D-Bus org.kde.serviceName.service file). Otherwise just use QProcess,
+     *   KIO::CommandLauncherJob or KIO::ApplicationLauncherJob.
      */
-    static int startServiceByDesktopName(const QString &_name, const QString &URL,
-                                         QString *error = nullptr, QString *serviceName = nullptr, int *pid = nullptr,
-                                         const QByteArray &startup_id = QByteArray(), bool noWait = false);
+    KSERVICE_DEPRECATED_VERSION_BELATED(5, 82, 5, 0, "See API documentation")
+    static int startServiceByDesktopName(const QString &_name,
+                                         const QString &URL,
+                                         QString *error = nullptr,
+                                         QString *serviceName = nullptr,
+                                         int *pid = nullptr,
+                                         const QByteArray &startup_id = QByteArray(),
+                                         bool noWait = false);
+#endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 82)
     /**
      * Starts a service based on the desktop name of the service.
      * E.g. "konqueror"
@@ -320,11 +382,18 @@ public:
      * @return an error code indicating success (== 0) or failure (> 0).
      * @deprecated since 5.0 use QDBusConnectionInterface::startService("org.kde.serviceName"),
      *   to start a unique application in order to make D-Bus calls to it (after ensuring that
-     *   it installs a D-Bus org.kde.serviceName.service file). Otherwise just use QProcess or KRun.
+     *   it installs a D-Bus org.kde.serviceName.service file). Otherwise just use QProcess,
+     *   KIO::CommandLauncherJob or KIO::ApplicationLauncherJob.
      */
-    static int startServiceByDesktopName(const QString &_name, const QStringList &URLs = QStringList(),
-                                         QString *error = nullptr, QString *serviceName = nullptr, int *pid = nullptr,
-                                         const QByteArray &startup_id = QByteArray(), bool noWait = false);
+    KSERVICE_DEPRECATED_VERSION_BELATED(5, 82, 5, 0, "See API documentation")
+    static int startServiceByDesktopName(const QString &_name,
+                                         const QStringList &URLs = QStringList(),
+                                         QString *error = nullptr,
+                                         QString *serviceName = nullptr,
+                                         int *pid = nullptr,
+                                         const QByteArray &startup_id = QByteArray(),
+                                         bool noWait = false);
+#endif
 
     /**
      * Starts a program via kdeinit.
@@ -343,8 +412,11 @@ public:
      *           "" ( empty string ) is the default
      * @return an error code indicating success (== 0) or failure (> 0).
      */
-    static int kdeinitExec(const QString &name, const QStringList &args = QStringList(),
-                           QString *error = nullptr, int *pid = nullptr, const QByteArray &startup_id = QByteArray());
+    static int kdeinitExec(const QString &name,
+                           const QStringList &args = QStringList(),
+                           QString *error = nullptr,
+                           int *pid = nullptr,
+                           const QByteArray &startup_id = QByteArray());
 
     /**
      * Starts a program via kdeinit and wait for it to finish.
@@ -363,8 +435,11 @@ public:
      *           "" ( empty string ) is the default
      * @return an error code indicating success (== 0) or failure (> 0).
      */
-    static int kdeinitExecWait(const QString &name, const QStringList &args = QStringList(),
-                               QString *error = nullptr, int *pid = nullptr, const QByteArray &startup_id = QByteArray());
+    static int kdeinitExecWait(const QString &name,
+                               const QStringList &args = QStringList(),
+                               QString *error = nullptr,
+                               int *pid = nullptr,
+                               const QByteArray &startup_id = QByteArray());
 
     /**
      * Ensures that kdeinit5 and klauncher are running.
@@ -380,10 +455,15 @@ Q_SIGNALS:
 
 private:
     int startServiceInternal(const char *_function,
-                             const QString &_name, const QStringList &URLs,
-                             QString *error, QString *serviceName, int *pid,
-                             const QByteArray &startup_id, bool noWait,
-                             const QString &workdir = QString());
+                             const QString &_name,
+                             const QStringList &URLs,
+                             QString *error,
+                             QString *serviceName,
+                             int *pid,
+                             const QByteArray &startup_id,
+                             bool noWait,
+                             const QString &workdir = QString(),
+                             const QStringList &envs = QStringList());
     static bool isMainThreadActive(QString *error = nullptr);
 
     KToolInvocationPrivate *const d;
@@ -391,4 +471,3 @@ private:
 };
 
 #endif
-
